/**
*
* Copyright (c) 2012 TOTAL IMMERSION, All Rights Reserved
* No part of this software and its documentation may be used, copied,
* modified, distributed and transmitted, in any form or by any means,
* without the prior written permission of TOTAL IMMERSION
*
* TOTAL IMMERSION, 22 rue Edouard Nieuport 92150 SURESNES FRANCE
* contact@t-immersion.com www.t-immersion.com
*
*/

#include "pxcsession.h"
#include "pxccapture.h"
#include "pxcsmartptr.h"
#include "util_render.h"
#include "util_capture.h"
#include "util_cmdline.h"

#include "pxcdcvtracker.h"
#include "dcvRender.h"

#include <string>
using namespace std;

const pxcCHAR* trackerPath = L"../tracker/tracker.xml";

bool computeModuleDirectory(void* iModule, pxcCHAR* oPath, DWORD pathLen)
{
	if (!::GetModuleFileNameW((HMODULE)iModule,oPath,pathLen)) {
		return false;
	}

	pxcCHAR* tmp = wcsrchr(oPath,L'\\');
	if (tmp==0) {
		return false;
	}
	*(tmp+1) = L'\0';
	return true;
}

int wmain(int argc, WCHAR* argv[]) 
{   
    // 
    PXCSmartPtr<PXCSession> session;
    pxcStatus sts=PXCSession_Create(&session);
	if (sts<PXC_STATUS_NO_ERROR) {
		wprintf(L"  \n");
		return 0;
	}

	session->LoadImplFromFile(TEXT("dfusion4PCSDK.dll"));
	{
		PXCSmartPtr<PXCDCVTracker>	tracker;
		PXCDCVTracker::ProfileInfo	pinfo;
		PXCSmartPtr<dcvRender>      renderer;
	
		session->CreateImpl(PXCDCVTracker::CUID,(void**)&tracker);
		if (!tracker)
			return 3;
		UtilCmdLine cmdl(session);
		if (!cmdl.Parse(L"-sdname-nframes",argc,argv)) return 3;

		UtilCapture capture(session);
		if (cmdl.m_sdname) capture.SetFilter(cmdl.m_sdname); /*L"Integrated Camera"*/
		//capture.SetFilter(L"DepthSense Device 325"); 

		tracker->QueryProfile(&pinfo);
		
		sts=capture.LocateStreams(&pinfo.inputs);
		if (sts<PXC_STATUS_NO_ERROR) {
			wprintf(L"    \n");
			return 0;
		}

		{
			PXCCapture::VideoStream * stream = capture.QueryVideoStream(0,0);
			PXCCapture::VideoStream::ProfileInfo vsinfo;
			stream->QueryProfile(&vsinfo);
			pinfo.width = vsinfo.imageInfo.width;
			pinfo.height= vsinfo.imageInfo.height;

			pxcCHAR path[MAX_PATH];
			computeModuleDirectory(0, (wchar_t*)path, MAX_PATH);
			wstring file = path;
			file += trackerPath;
			pxcStatus st = tracker->SetProfile(file.c_str(), &pinfo);
			if (st<PXC_STATUS_NO_ERROR) {
				wprintf(L"   \n");
				return 0;
			}
		}

		//  
		renderer=new dcvRender(L"D'Fusion CV Detection Sample");
		if (!renderer) {
			wprintf(L"   \n");
			return 0;
		}

		renderer->addData(tracker);


		for (int fnum=0;fnum<cmdl.m_nframes;fnum++) 
		{
			PXCSmartArray<PXCImage> images;
			PXCSmartSP sp1, sp2;

			/*      */
			sts=capture.ReadStreamAsync(images, &sp1);
			if (sts<PXC_STATUS_NO_ERROR) break;
			sts=tracker->ProcessImageAsync(images, &sp2);
			if (sts<PXC_STATUS_NO_ERROR) break;
			

			if(sp2) 
				sp2->Synchronize();

			if (!renderer->RenderFrame(images[0])) 
				break;
		}
	}
	
	session->UnloadImplFromFile(TEXT("dfusion4PCSDK.dll"));

    return(0);
}

